#include <QApplication>
#include <QDebug>
#include <QApplication>
#include <QRemoteObjectNode>
#include <QAbstractItemModelReplica>
#include <QStandardItemModel>
#include <QStandardItem>
#include <QSortFilterProxyModel>
#include <qboxlayout.h>
#include <qlineedit.h>
#include <QTableView>
#include <QLabel>
#include <QProcess>
#include <qmainwindow.h>
#include <QPushButton>
#include <QIdentityProxyModel>

class W : public QWidget
{
public:

};

class MW : public QMainWindow
{
public:
    MW()
    {
        setMouseTracking(true);
    }

    // QWidget interface
protected:
    void mouseMoveEvent(QMouseEvent *event) override
    {
        qDebug("move");
    }
};

int main(int argc, char** argv)
{
    QLoggingCategory::setFilterRules("qt.remoteobjects=true");
    QApplication app(argc, argv);

    bool client = app.arguments().contains("-c");

    if (client) {
        QRemoteObjectNode node(QUrl(QStringLiteral("local:registry")));
        node.setHeartbeatInterval(1000);
        QTableView* view = new QTableView;
        view->setWindowTitle(QStringLiteral("RemoteView"));
        view->resize(800, 600);
        QScopedPointer<QAbstractItemModelReplica> model(node.acquireModel(QStringLiteral("RemoteModel")));
        view->setModel(model.data());
        view->setSelectionModel(model->selectionModel());

        QWidget w;
        w.setLayout(new QVBoxLayout);
        auto label = new QLabel(&w);
        w.layout()->addWidget(label);
        w.layout()->addWidget(view);

        QObject::connect(model->selectionModel(), &QItemSelectionModel::currentChanged, [label](QModelIndex cur, QModelIndex old) {
            label->setText("Current row #: " + QString::number(cur.row()));
        });

        w.show();

        return app.exec();
    }
    else {
        QStandardItemModel sourceModel;
        for (int i = 1; i < 50; ++i)
            sourceModel.appendRow(new QStandardItem("Item " + QString::number(i)));

        QRemoteObjectRegistryHost node(QUrl(QStringLiteral("local:registry")));

        QSortFilterProxyModel sfpm;
        sfpm.setSourceModel(&sourceModel);
        auto sm = new QItemSelectionModel(&sfpm);

        QVector<int> roles;
        roles << Qt::DisplayRole;

        QRemoteObjectHost node2(QUrl(QStringLiteral("local:replica")), QUrl(QStringLiteral("local:registry")));
        node2.enableRemoting(&sfpm, QStringLiteral("RemoteModel"), roles, sm);

        QWidget w;
        w.setLayout(new QVBoxLayout);
        QTableView *view = new QTableView(&w);
        auto le = new QLineEdit(&w);

        QObject::connect(le, &QLineEdit::textChanged, [&sfpm](QString text){
            sfpm.setFilterRegExp(text);
        });
        w.layout()->addWidget(le);

        auto label = new QLabel(&w);
        w.layout()->addWidget(label);

        QObject::connect(sm, &QItemSelectionModel::currentChanged, [label](QModelIndex cur, QModelIndex old) {
            label->setText("Current row #: " + QString::number(cur.row()));
        });

        w.layout()->addWidget(view);
        view->setWindowTitle(QStringLiteral("SourceView"));
        view->setModel(&sfpm);
        view->setSelectionModel(sm);
        view->setSortingEnabled(true);
        w.resize(800, 600);
        w.show();

        QProcess::startDetached(app.applicationFilePath(), QStringList() << "-c");
        return app.exec();
    }

}
